<?php

class MapConfig {
  
  var $dom;
  var $root;
  var $custom_elements;
  
  function MapConfig($node) {
    $this->dom = new dom_DOMDocument('1.0', 'UTF-8');
    
    $this->root = $this->init($node);
    
    // append to dom
    dom_appendChild($this->dom->doc, $this->root );
  }
  
  /**
   * create an element of a given class, include the custom scriptFile if needed,
   * and set the id attribute
   */
  function createElement($className, $type = 'widget', $id = NULL) {
    static $custom_basedir;
    
    if (is_null($this->custom_elements)) {
      $this->custom_elements = explode(',', variable_get('mapbuilder_custom_elements', ''));
      $custom_basedir = config_path_to_url(variable_get('mapbuilder_custom_path',DEFAULT_MAPBUILDER_CUSTOM_PATH));
    }
    $node = $this->dom->createElement($className);
    if ($id) {
      dom_setAttribute($node, 'id', $id);
    }
    if (in_array($className, $this->custom_elements)) {
      dom_appendChild($node, $this->dom->createElement('scriptFile', $custom_basedir."lib/$type/$className.js"));
    }
    return $node;
  }
  
  function init($node) {
    $custom_elements = explode(',', variable_get('mapbuilder_custom_elements', ''));

    $params = $_GET;
    unset( $params['q'] );
    
    if (isset($params['blocks'])) {
      $blocks = explode(',', $params['blocks']);
      unset($params['blocks']);
    }
    else {
      $blocks = array();
    }
    
    $htmlTagId = 'mainMapPane_' . $node->vid;
    $mapContainerId = 'mainMapContainer';//_' . $node->vid; bug http://jira.codehaus.org/browse/MAP-169
    
    $root = $this->dom->createElement( 'MapbuilderConfig' );
    dom_setAttribute($root, 'id', 'basicMap' );
    dom_setAttribute($root, 'version', '0.2.1' );
    dom_setAttribute($root, 'xmlns', "http://mapbuilder.sourceforge.net/mapbuilder" );
    dom_setAttribute($root, 'xmlns:xsi', "http://www.w3.org/2001/XMLSchema-instance" );
    dom_setAttribute($root, 'xsi:schemaLocation', "http://mapbuilder.sourceforge.net/mapbuilder ../../lib/schemas/config.xsd" );

    // node info
    //dom_appendChild($root, $this->dom->createComment('drupal node: '.var_export($node,true)));
    dom_appendChild($root, $this->dom->createComment('This file is generated by the drupal mapbuilder module.'));
    
    // determine if latlon is displayed or x/y
    $showLatLong = false;
    if ($node->modelVid != 0) {
      // get SRS from modelVid
      $bbox = db_result(db_query("select bbox from {mapbuilder_context} where vid = %d", $node->modelVid));
      $tmp = explode(',',$bbox);
      if ($tmp[0] == 'EPSG:4326')
        $showLatLong = true;
    }
    
    $custom_basedir = config_path_to_url(variable_get('mapbuilder_custom_path',DEFAULT_MAPBUILDER_CUSTOM_PATH));
    
    // (1) models
    $models = $this->dom->createElement( 'models' );
    dom_appendChild($root, $models );

    // add context (type must correspond with mappane)
    $context = $this->createElement('Context', 'model', 'mainMap_'.$node->vid);
    //$context = $this->dom->createElement( 'OwsContext' );
    dom_appendChild($models, $context );
    
    $item = $this->dom->createElement( 'defaultModelUrl' );
    dom_appendChild($item, $this->dom->createTextNode( config_path_to_url($node->modelUrl) . param_string($params) ) );
    dom_appendChild($context, $item );
    
    // context widgets
    $widgets = dom_appendChild($context, $this->dom->createElement( 'widgets' ) );
    
    // add MapPane OR MapPaneOL
    $mapPane = dom_appendChild($widgets, $this->createElement('MapPane', 'widget', 'mainMapWidget_' . $node->vid) );
    //$mapPane = dom_appendChild($widgets, $this->dom->createElement( 'MapPaneOL' ) );
    dom_appendChild($mapPane, $this->dom->createElement( 'htmlTagId', $htmlTagId ) );
    dom_appendChild($mapPane, $this->dom->createElement( 'mapContainerId', $mapContainerId ) );
    
    // Legend widget
    if (in_array('legend', $blocks)) {
      $item = dom_appendChild($widgets, $this->createElement('Legend', 'widget', 'legend') );
      // for mapbuilder >= 1.5
//      if ('1.5-alpha1' == variable_get('mapbuilder_version',DEFAULT_MAPBUILDER_VERSION)) {
//        $item = dom_appendChild($widgets, $this->dom->createElement( 'LegendGraphic' ) );
//        dom_setAttribute($item, 'id', 'legendGraphic' );
//      }
    }
    else {
      $item = dom_appendChild($widgets, $this->createElement('Legend', 'widget', 'legend_' . $node->vid) );
    }
    
    // Loading widget
    $item = dom_appendChild($widgets, $this->dom->createElement( 'Loading' ) );
    dom_setAttribute($item, 'id', 'loading_' . $node->vid );
    
    // draw rectangle
    $item = dom_appendChild($widgets, $this->dom->createElement( 'AoiBoxDHTML' ) );
    dom_setAttribute($item, 'id', 'aoiBoxMain_' . $node->vid );
    dom_appendChild($item, $this->dom->createElement( 'htmlTagId', $htmlTagId ) );
    dom_appendChild($item, $this->dom->createElement( 'mapContainerId', $mapContainerId ) );
    dom_appendChild($item, $this->dom->createElement( 'lineWidth', 2 ) );
    dom_appendChild($item, $this->dom->createElement( 'lineColor', '#FF0000' ) );
    dom_appendChild($item, $this->dom->createElement( 'crossSize', 5 ) );
    
    if (in_array('scalebar', $blocks)) {
      // scalebar
      $item = dom_appendChild($widgets, $this->dom->createElement( 'MapScaleBar' ) );
      dom_setAttribute($item, 'id', 'scalebar');
      dom_appendChild($item, $this->dom->createElement('displaySystem', 'metric'));
      dom_appendChild($item, $this->dom->createElement('align', 'left'));
    }
    
    // mapscaletext (commented out in mapbuilder_generate_html function)
    //$item = dom_appendChild($widgets, $this->dom->createElement('MapScaleText'));
    //dom_setAttribute($item, 'id', 'mapScaleText_' . $node->vid);
    
    // cursortrack
    $item = dom_appendChild($widgets, $this->dom->createElement('CursorTrack'));
    dom_setAttribute($item, 'id', 'cursorTrack_' . $node->vid);
    dom_appendChild($item, $this->dom->createElement('mouseHandler', 'mainMap_' . $node->vid));
    if ($showLatLong) {
      dom_appendChild($item, $this->dom->createElement('showLatLong', 'true'));
    }
    else {
      dom_appendChild($item, $this->dom->createElement('showLatLong', 'false'));
      dom_appendChild($item, $this->dom->createElement('showXY', 'true'));
      dom_appendChild($item, $this->dom->createElement('precision', '0'));
    }
    
    // context tools
    $tools = dom_appendChild($context, $this->dom->createElement( 'tools' ) );
    $item = dom_appendChild($tools, $this->dom->createElement( 'AoiMouseHandler' ) );
    dom_setAttribute($item, 'id', 'mainAoi_' . $node->vid );
    $item = dom_appendChild($tools, $this->dom->createElement( 'History' ) );
    dom_setAttribute($item, 'id', 'extentHistory_' . $node->vid );
    $item = dom_appendChild($tools, $this->dom->createElement( 'DragPanHandler' ) );
    dom_setAttribute($item, 'id', 'mainDragPan_' . $node->vid );
    dom_appendChild($item, $this->dom->createElement( 'enabled', 'false' ) );
    
    // (2) main widgets
    $widgets = dom_appendChild($root, $this->dom->createElement( 'widgets' ) );

    $item = dom_appendChild($widgets, $this->dom->createElement( 'ZoomIn' ) );
    dom_setAttribute($item, 'id', 'zoomIn_' . $node->vid );
    dom_appendChild($item, $this->dom->createElement( 'class', 'RadioButton' ) );
    dom_appendChild($item, $this->dom->createElement( 'targetModel', 'mainMap_' . $node->vid ) );
    dom_appendChild($item, $this->dom->createElement( 'buttonBar', 'mainButtonBar_' . $node->vid ) );
    dom_appendChild($item, $this->dom->createElement( 'selected', true ) );
    dom_appendChild($item, $this->dom->createElement( 'mouseHandler', 'mainAoi_' . $node->vid ) );
    dom_appendChild($item, $this->dom->createElement( 'enabledSrc', '/images/ZoomInEnable.png' ) );
    dom_appendChild($item, $this->dom->createElement( 'disabledSrc', '/images/ZoomInDisable.png' ) );
    
    $item = dom_appendChild($widgets, $this->dom->createElement( 'ZoomOut' ) );
    dom_setAttribute($item, 'id', 'zoomOut_' . $node->vid );
    dom_appendChild($item, $this->dom->createElement( 'class', 'RadioButton' ) );
    dom_appendChild($item, $this->dom->createElement( 'targetModel', 'mainMap_' . $node->vid ) );
    dom_appendChild($item, $this->dom->createElement( 'buttonBar', 'mainButtonBar_' . $node->vid ) );
    dom_appendChild($item, $this->dom->createElement( 'mouseHandler', 'mainAoi_' . $node->vid ) );
    dom_appendChild($item, $this->dom->createElement( 'enabledSrc', '/images/ZoomOutEnable.png' ) );
    dom_appendChild($item, $this->dom->createElement( 'disabledSrc', '/images/ZoomOutDisable.png' ) );
    
    $item = dom_appendChild($widgets, $this->dom->createElement( 'DragPan' ) );
    dom_setAttribute($item, 'id', 'dragPan_' . $node->vid );
    dom_appendChild($item, $this->dom->createElement( 'class', 'RadioButton' ) );
    dom_appendChild($item, $this->dom->createElement( 'targetModel', 'mainMap_' . $node->vid ) );
    dom_appendChild($item, $this->dom->createElement( 'buttonBar', 'mainButtonBar_' . $node->vid ) );
    dom_appendChild($item, $this->dom->createElement( 'mouseHandler', 'mainDragPan_' . $node->vid ) );
    dom_appendChild($item, $this->dom->createElement( 'enabledSrc', '/images/PanEnable.png' ) );
    dom_appendChild($item, $this->dom->createElement( 'disabledSrc', '/images/PanDisable.png' ) );
    
    $item = dom_appendChild($widgets, $this->dom->createElement( 'Reset' ) );
    dom_setAttribute($item, 'id', 'reset_' . $node->vid );
    dom_appendChild($item, $this->dom->createElement( 'class', 'Button' ) );
    dom_appendChild($item, $this->dom->createElement( 'targetModel', 'mainMap_' . $node->vid ) );
    dom_appendChild($item, $this->dom->createElement( 'buttonBar', 'mainButtonBar_' . $node->vid ) );
    dom_appendChild($item, $this->dom->createElement( 'disabledSrc', '/images/ResetExtentDisable.png' ) );
    
    $item = dom_appendChild($widgets, $this->dom->createElement( 'Back' ) );
    dom_setAttribute($item, 'id', 'back_' . $node->vid );
    dom_appendChild($item, $this->dom->createElement( 'class', 'Button' ) );
    dom_appendChild($item, $this->dom->createElement( 'targetModel', 'mainMap_' . $node->vid ) );
    dom_appendChild($item, $this->dom->createElement( 'buttonBar', 'mainButtonBar_' . $node->vid ) );
    dom_appendChild($item, $this->dom->createElement( 'disabledSrc', '/images/UndoDisable.png' ) );
  
    $item = dom_appendChild($widgets, $this->dom->createElement( 'Forward' ) );
    dom_setAttribute($item, 'id', 'forward_' . $node->vid );
    dom_appendChild($item, $this->dom->createElement( 'class', 'Button' ) );
    dom_appendChild($item, $this->dom->createElement( 'targetModel', 'mainMap_' . $node->vid ) );
    dom_appendChild($item, $this->dom->createElement( 'buttonBar', 'mainButtonBar_' . $node->vid ) );
    dom_appendChild($item, $this->dom->createElement( 'disabledSrc', '/images/RedoDisable.png' ) );

    if (in_array('featurelist', $blocks)) {
      $item = dom_appendChild($widgets, $this->createElement('WfsGetFeature', 'widget', 'wfsGetFeature_' . $node->vid) );
      dom_appendChild($item, $this->dom->createElement( 'class', 'RadioButton' ) );
      dom_appendChild($item, $this->dom->createElement( 'targetModel', 'mainMap_' . $node->vid ) );
      dom_appendChild($item, $this->dom->createElement( 'buttonBar', 'mainButtonBar_' . $node->vid ) );
      dom_appendChild($item, $this->dom->createElement( 'mouseHandler', 'mainAoi_' . $node->vid ) );
      dom_appendChild($item, $this->dom->createElement( 'enabledSrc', '/images/QueryEnable.png' ) );
      dom_appendChild($item, $this->dom->createElement( 'disabledSrc', '/images/QueryDisable.png' ) );
      
      dom_appendChild($item, $this->dom->createElement('transactionResponseModel', 'featureCollection'));
      //$url = '../../../geoserver/wfs';
      //dom_appendChild($item, $this->dom->createElement('webServiceUrl'));//, $url));
      //dom_appendChild($item, $this->dom->createElement('typeName'));//, 'ws'));
      dom_appendChild($item, $this->dom->createElement('tolerance', '5'));
      $maxFeatures = variable_get('mapbuilder_maxfeatures', DEFAULT_MAPBUILDER_MAXFEATURES);
      if (is_numeric($maxFeatures) && $maxFeatures > 0)
        dom_appendChild($item, $this->dom->createElement('maxFeatures', $maxFeatures));
      
      $model = $this->dom->createElement('FeatureCollection');
      dom_setAttribute($model, 'id', 'featureCollection');
      dom_appendChild($models, $model);
      $fwidgets = dom_appendChild($model, $this->dom->createElement('widgets'));
      $item = dom_appendChild($fwidgets, $this->dom->createElement('GmlRendererWZ'));
      dom_setAttribute($item, 'id', 'testGmlRenderer');
      dom_appendChild($item, $this->dom->createElement('htmlTagId', $htmlTagId));
      dom_appendChild($item, $this->dom->createElement('mapContainerId', $mapContainerId));
      dom_appendChild($item, $this->dom->createElement('targetModel', 'mainMap_' . $node->vid));
      dom_appendChild($item, $this->dom->createElement('lineWidth', 3));
      dom_appendChild($item, $this->dom->createElement('lineColor', '#FF0000'));
      dom_appendChild($item, $this->dom->createElement('pointDiameter', '10'));
      $item = dom_appendChild($fwidgets, $this->createElement('FeatureList', 'widget', 'featureList'));
      if (in_array('FeatureList', $this->custom_elements)) {
        // NB: the widget doesn't automatically load the stylesheet next to it
        dom_appendChild($item, $this->dom->createElement('stylesheet', $custom_basedir.'lib/widget/FeatureList.xsl'));
      }
      dom_appendChild($model, $this->dom->createElement('namespace', "xmlns:gml='http://www.opengis.net/gml' xmlns:wfs='http://www.opengis.net/wfs' xmlns:topp='http://www.openplans.org/topp'"));
    }
    
    if (in_array('featureinfo', $blocks)) {
      // feature info
      $item = dom_appendChild($widgets, $this->createElement('GetFeatureInfo', 'widget', 'getFeatureInfo_' . $node->vid));
      //dom_appendChild($item, $this->dom->createElement('debug'));
      dom_appendChild($item, $this->dom->createElement( 'class', 'RadioButton' ) );
      dom_appendChild($item, $this->dom->createElement( 'targetModel', 'mainMap_' . $node->vid ) );
      dom_appendChild($item, $this->dom->createElement( 'buttonBar', 'mainButtonBar_' . $node->vid ) );
      dom_appendChild($item, $this->dom->createElement( 'mouseHandler', 'mainAoi_' . $node->vid ) );
      dom_appendChild($item, $this->dom->createElement( 'enabledSrc', '/images/QueryEnable.png' ) );
      dom_appendChild($item, $this->dom->createElement( 'disabledSrc', '/images/QueryDisable.png' ) );
      
      $model = $this->dom->createElement('Model');
      dom_setAttribute($model, 'id', 'featureInfoResponse');
      dom_setAttribute($model, 'template', 'true');
      dom_appendChild($models, $model);
      $item = dom_appendChild($model, $this->dom->createElement('widgets'));
      $item = dom_appendChild($item, $this->createElement('FeatureInfo', 'widget', 'features'));
      //dom_setAttribute($item, 'id', 'features');
      dom_appendChild($item, $this->dom->createElement('htmlTagId', 'workspace'));
      dom_appendChild($item, $this->dom->createElement('outputNodeid', 'workspaceCanvas'));
      dom_appendChild($model, $this->dom->createElement('namespace', "xmlns:gml='http://www.opengis.net/gml' xmlns:wfs='http://www.opengis.net/wfs' xmlns:topp='http://www.openplans.org/topp'"));
      
      $item = dom_appendChild($tools, $this->createElement('WebServiceRequest', 'tool', 'featureInfoController'));
      //dom_setAttribute($item, 'id', 'featureInfoController');
      dom_appendChild($item, $this->dom->createElement('targetModel', 'featureInfoResponse'));
      dom_appendChild($item, $this->dom->createElement('requestName', 'wms:GetFeatureInfo'));
      dom_appendChild($item, $this->dom->createElement('infoFormat', 'text/html'));
      //dom_appendChild($item, $this->dom->createElement('infoFormat', 'application/vnd.ogc.gml'));
      dom_appendChild($item, $this->dom->createElement('featureCount', '1'));
      
      // add to context element
      dom_appendChild($context, $this->dom->createElement('nodeSelectXpath', '/wmc:ViewContext/wmc:LayerList/wmc:Layer'));
    }
    
    $item = dom_appendChild($widgets, $this->dom->createElement( 'Save' ) );
    dom_setAttribute($item, 'id', 'save_' . $node->vid );
    dom_appendChild($item, $this->dom->createElement( 'class', 'Button' ) );
    dom_appendChild($item, $this->dom->createElement( 'targetModel', 'mainMap_' . $node->vid ) );
    dom_appendChild($item, $this->dom->createElement( 'buttonBar', 'mainButtonBar_' . $node->vid ) );
    dom_appendChild($item, $this->dom->createElement( 'disabledSrc', '/images/SaveDisable.png' ) );
    dom_appendChild($item, $this->dom->createElement( 'popupWindowName', 'SaveContextXML' ) );
    
    if (in_array('overview', $blocks)) {
      // add locator context if an overview map is available
      $context = $this->dom->createElement( 'Context' );
      dom_setAttribute($context, 'id', 'locator' );
      dom_appendChild($models, $context );
      $item = $this->dom->createElement( 'defaultModelUrl' );
      dom_appendChild($item, $this->dom->createTextNode( config_path_to_url($node->locatorUrl) . param_string($params) ) );
      dom_appendChild($context, $item );
      $widgets = dom_appendChild($context, $this->dom->createElement( 'widgets' ) );
      $mapPane = dom_appendChild($widgets, $this->createElement('MapPane', 'widget', 'locatorWidget') );
      dom_appendChild($mapPane, $this->dom->createElement( 'htmlTagId', 'locatorMap' ) );
      dom_appendChild($mapPane, $this->dom->createElement( 'mapContainerId', 'locatorContainer' ) );
      $fixedWidth = variable_get('mapbuilder_locator_width',DEFAULT_MAPBUILDER_LOCATOR_WIDTH);
      if (intval($fixedWidth) > 0) {
        dom_appendChild($mapPane, $this->dom->createElement( 'fixedWidth', $fixedWidth ) );
      }
      dom_appendChild($mapPane, $this->dom->createElement( 'extraAttributes', 'false' ) );
    
      $item = dom_appendChild($widgets, $this->dom->createElement( 'AoiBoxDHTML' ) );
      dom_setAttribute($item, 'id', 'aoiBoxLocator' );
      dom_appendChild($item, $this->dom->createElement( 'htmlTagId', 'locatorMap' ) );
      dom_appendChild($item, $this->dom->createElement( 'mapContainerId', 'locatorContainer' ) );
      dom_appendChild($item, $this->dom->createElement( 'lineWidth', 1 ) );
      dom_appendChild($item, $this->dom->createElement( 'lineColor', '#FF0000' ) );
      dom_appendChild($item, $this->dom->createElement( 'crossSize', 5 ) );
    
      $tools = dom_appendChild($context, $this->dom->createElement( 'tools' ) );
      $item = dom_appendChild($tools, $this->dom->createElement( 'AoiMouseHandler' ) );
      dom_setAttribute($item, 'id', 'locatorAoi' );
      $item = dom_appendChild($tools, $this->dom->createElement( 'ZoomToAoi' ) );
      dom_setAttribute($item, 'id', 'locatorZoomToAoi' );
      dom_appendChild($item, $this->dom->createElement( 'targetModel', 'mainMap_' . $node->vid ) );
    }
    
    // (3) todo: skindir can contain an absolute path!!
    dom_appendChild($root, $this->dom->createElement('skinDir', config_path_to_url(variable_get('mapbuilder_skindir', DEFAULT_MAPBUILDER_SKINDIR ))));
    
//  global $locale;
//  dom_appendChild($root, $dom->createElement( 'languages', $locale ) );
    dom_appendChild($root, $this->dom->createElement( 'widgetTextUrl', 'widgetText.xml' ) );
    dom_appendChild($root, $this->dom->createComment( 'relative to the skinDir + language subdir' ) );
    
    dom_appendChild($root, $this->dom->createElement( 'proxyUrl'
      , base_path() . drupal_get_path('module','mapbuilder') . '/server/proxy.php' ) );
    dom_appendChild($root, $this->dom->createElement( 'serializeUrl'
      , base_path() . drupal_get_path('module','mapbuilder') . '/server/serializer.php'
      . '?file_directory_temp=' . variable_get('file_directory_temp','files')
      . '&amp;base_path=' . base_path()
      ) );

    return $root;
  }
  
  function getXML() {
    return $this->dom->saveXML();
  }
  
}

?>
