<?php
/**
 *
 */

/**
 *
 */
class MapContext {
  
  var $dom;
  var $root;
  var $general;
  var $window;
  var $list;
  
  function MapContext( $node ) {
    $this->dom = new dom_DOMDocument('1.0', 'UTF-8');
    
    $this->root = $this->init();
    
    dom_appendChild($this->general, $this->dom->createElement( 'Title', $node->title ) );
    
    $this->window = dom_appendChild($this->general, $this->dom->createElement( 'Window' ) );
  }
  
  function init() {
    $this->root = $root = $this->dom->createElement( 'ViewContext' );
    dom_setAttribute($root, 'version', '1.0.0' );
    dom_setAttribute($root, 'xmlns', "http://www.opengis.net/context" );
    dom_setAttribute($root, 'xmlns:xlink', "http://www.w3.org/1999/xlink" );
    dom_setAttribute($root, 'xmlns:xsi', "http://www.w3.org/2001/XMLSchema-instance" );

    dom_setAttribute($root, 'xsi:schemaLocation', "http://www.opengis.net/context http://schemas.opengis.net/context/1.0.0/context.xsd" );

    $this->general = dom_appendChild($this->root, $this->dom->createElement( 'General' ) );
    $this->list = dom_appendChild($this->root, $this->dom->createElement( 'LayerList' ) );
    
    return dom_appendChild($this->dom->doc, $root );
  }
  
  function setWindow($width, $height) {
    dom_setAttribute($this->window, 'width', $width );
    dom_setAttribute($this->window, 'height', $height );
  }
  
  function setBoundingBox( $bboxStr ) {
    $bbox = explode( ',', $bboxStr );
    $bboxNode = dom_appendChild($this->general, $this->dom->createElement( 'BoundingBox' ) );
    dom_setAttribute($bboxNode, 'SRS', $bbox[0] );
    dom_setAttribute($bboxNode, 'minx', $bbox[1] );
    dom_setAttribute($bboxNode, 'miny', $bbox[2] );
    dom_setAttribute($bboxNode, 'maxx', $bbox[3] );
    dom_setAttribute($bboxNode, 'maxy', $bbox[4] );
  }
  
  function addLayer($layerData) {
    $layer = $this->dom->createElement( 'Layer' );
    dom_setAttribute($layer, 'hidden', $layerData['hidden'] );
    dom_setAttribute($layer, 'queryable', $layerData['queryable'] );
    
    dom_appendChild($layer, $this->dom->createElement( 'Name', $layerData['names'] ) );
    dom_appendChild($layer, $this->dom->createElement( 'Title', $layerData['layer_title'] ) );
    
    $server = dom_appendChild($layer, $this->dom->createElement( 'Server' ) );
    dom_setAttribute($server, 'version', $layerData['version'] );
    dom_setAttribute($server, 'service', "OGC:WMS" );
    $onlineResource = dom_appendChild($server, $this->dom->createElement( 'OnlineResource' ) );
    dom_setAttribute($onlineResource, 'xlink:type', 'simple' );
    dom_setAttribute($onlineResource, 'xlink:href', $layerData['href'] );
    
    // set default image format
    $item = dom_appendChild($layer, $this->dom->createElement('FormatList'));
    $item = dom_appendChild($item, $this->dom->createElement('Format',variable_get('mapbuilder_image_format',DEFAULT_MAPBUILDER_IMAGE_FORMAT)));
    dom_setAttribute($item, 'current', '1');
    
    // legend graphic
    $item = dom_appendChild($layer, $this->dom->createElement('StyleList'));
    $style = dom_appendChild($item, $this->dom->createElement('Style'));
    dom_setAttribute($style, 'current', '1');
    dom_appendChild($style, $this->dom->createElement('Name', 'default'));
    dom_appendChild($style, $this->dom->createElement('Title', 'default'));
    if ($layerData['legendurl'] != '') {
      $legend_url = $layerData['legendurl'];
      
      // todo: derive image format from legend_url
      $legend_format = variable_get('mapbuilder_image_format',DEFAULT_MAPBUILDER_IMAGE_FORMAT);
    }
    else {
      // maybe the legend can be generated by the service (assume umn mapserver type)
      $legend_format = variable_get('mapbuilder_image_format',DEFAULT_MAPBUILDER_IMAGE_FORMAT);
      $legend_url = $layerData['href']
      . '&version=' . $layerData['version']
      . '&request=GetLegendGraphic&layer=' . $layerData['names']
      . '&format=' . $legend_format;
    }
    
    $url = dom_appendChild($style, $this->dom->createElement('LegendURL'));
    dom_setAttribute($url, 'format', $legend_format);
    $rsc = dom_appendChild($url, $this->dom->createElement('OnlineResource'));
    dom_setAttribute($rsc, 'xlink:type', 'simple');
    dom_setAttribute($rsc, 'xlink:href', $legend_url);
    
    dom_appendChild($this->list, $layer );
  }
  
  function getXML() {
    return $this->dom->saveXML();
  }
}

/**
 *
 */
class OwsContext extends MapContext {
  
  function OwsContext( $node ) {
    parent::MapContext( $node );
  }
  
  function init() {
    $this->root = $root = $this->dom->createElement('OWSContext');
    dom_setAttribute($root, 'version', '0.0.13' );
    dom_setAttribute($root, 'xmlns', "http://www.opengis.net/context" );
    dom_setAttribute($root, 'xmlns:xlink', "http://www.w3.org/1999/xlink" );
    dom_setAttribute($root, 'xmlns:xsi', "http://www.w3.org/2001/XMLSchema-instance" );

    dom_setAttribute($root, 'xmlns:sld', "http://www.opengis.net/sld");
    dom_setAttribute($root, 'xmlns:ogc', "http://www.opengis.net/ogc");
    dom_setAttribute($root, 'xmlns:ows', "http://www.opengis.net/ows");
    dom_setAttribute($root, 'xmlns:param', "http;//www.opengis.net/param");
    dom_setAttribute($root, 'xsi:schemaLocation', "http://www.opengis.net/oc oc_0_0_13.xsd");

    $this->general = dom_appendChild($this->root, $this->dom->createElement( 'General' ) );
    $this->list = dom_appendChild($this->root, $this->dom->createElement( 'ResourceList' ) );
    
    return dom_appendChild($this->dom->doc, $root );
  }
  
  function setBoundingBox( $bboxStr ) {
    $bbox = explode( ',', $bboxStr );
    
    $node = dom_appendChild($this->general, $this->dom->createElement( 'ows:BoundingBox' ) );
    dom_setAttribute($node, 'crs', $bbox[0] );
    dom_appendChild($node, $this->dom->createElement('ows:LowerCorner', $bbox[1].' '.$bbox[2]) );
    dom_appendChild($node, $this->dom->createElement('ows:UpperCorner', $bbox[3].' '.$bbox[4]) );
  }
  
}

?>
